@@ -26,7 +26,7 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-$Id: GraphOld.pm 4599 2012-01-20 16:08:39Z steve.schnepp $
+$Id$
 
 =end comment
 
@@ -89,6 +89,7 @@ my $log_file       = undef;
 my $skip_locking   = 0;
 my $skip_stats     = 0;
 my $stdout         = 0;
+my $force_run_as_root = 0;
 my $conffile       = $Munin::Common::Defaults::MUNIN_CONFDIR . "/munin.conf";
 my $libdir         = $Munin::Common::Defaults::MUNIN_LIBDIR;
 # Note: Nothing by default is more convenient and elliminates code while
@@ -106,7 +107,7 @@ my %draw           = (
 my %init_draw = %draw;
 my $pinpoint = {};
 
-my ($size_x, $size_y);
+my ($size_x, $size_y, $full_size_mode, $only_graph);
 my ($lower_limit, $upper_limit);
 
 my %PALETTE;    # Hash of available palettes
@@ -235,6 +236,8 @@ sub graph_startup {
 
     $size_x 	    = undef;
     $size_y         = undef;
+    $full_size_mode = undef;
+    $only_graph     = undef;
     $lower_limit    = undef;
     $upper_limit    = undef;
 
@@ -254,6 +257,7 @@ sub graph_startup {
                 "only-fqn=s"    => sub{ $only_fqn = process_fqn(@_); },
                 "config=s"      => \$conffile,
                 "stdout!"       => \$stdout,
+                "force-run-as-root!" => \$force_run_as_root,
                 "day!"          => \$draw{'day'},
                 "week!"         => \$draw{'week'},
                 "month!"        => \$draw{'month'},
@@ -263,6 +267,8 @@ sub graph_startup {
                 "sumyear!"      => \$draw{'sumyear'},
 		"size_x=i"      => \$size_x,
 		"size_y=i"      => \$size_y,
+		"full_size_mode!"=> \$full_size_mode,
+		"only_graph!"=> \$only_graph,
 		"upper_limit=s" => \$upper_limit,
 		"lower_limit=s" => \$lower_limit,
                 "list-images!"  => \$list_images,
@@ -285,7 +291,7 @@ sub graph_startup {
       print_usage_and_exit();
     }
 
-    exit_if_run_by_super_user();
+    exit_if_run_by_super_user() unless $force_run_as_root;
 
     # Only read $config once (thx Jani M.)
     #
@@ -327,7 +333,7 @@ sub graph_startup {
 sub graph_check_cron {
 
     # Are we running from cron and do we have matching graph_strategy
-    if (&munin_get($config, "graph_strategy", "cron") ne "cron" and $cron) {
+    if (&munin_get($config, "graph_strategy", "cgi") ne "cron" and $cron) {
 
         # Strategy mismatch: We're run from cron, but munin.conf says
         # we use dynamic graph generation
@@ -351,6 +357,8 @@ sub graph_main {
     # Reset an eventual custom size
     $size_x 	    = undef;
     $size_y         = undef;
+    $full_size_mode = undef;
+    $only_graph     = undef;
     $lower_limit    = undef;
     $upper_limit    = undef;
     $pinpoint       = undef;
@@ -375,6 +383,8 @@ sub graph_main {
 
 		"size_x=i"      => \$size_x,
 		"size_y=i"      => \$size_y,
+		"full_size_mode!"=> \$full_size_mode,
+		"only_graph!"   => \$only_graph,
 		"upper_limit=s" => \$upper_limit,
 		"lower_limit=s" => \$lower_limit,
 	    );
@@ -534,6 +544,9 @@ sub get_header {
     push @$result, "--height", ($size_y || munin_get($service, "graph_height", "175"));
     push @$result, "--width",  ($size_x || munin_get($service, "graph_width",  "400"));
 
+    push @$result, "--full-size-mode" if ($full_size_mode);
+    push @$result, "--only-graph" if ($only_graph);
+
     push @$result,"--rigid" if (defined $lower_limit || defined $upper_limit);
 
     push @$result, "--imgformat", "PNG";
@@ -1079,6 +1092,10 @@ sub process_service {
         }
 
         my $colour = munin_get($field, "colour");
+
+	if ($colour && $colour =~ /^COLOUR(\d+)$/) {
+		$colour = $COLOUR[$1 % @COLOUR];
+	}
 	
 	# Select a default colour if no explict one
 	$colour ||= ($single_value) ? $single_colour : $COLOUR[$field_count % @COLOUR];
