@@ -1,7 +1,7 @@
 package Munin::Master::Utils;
 # -*- cperl -*-
 
-# $Id: Utils.pm 4445 2011-10-18 08:07:12Z steve.schnepp $
+# $Id$
 
 use strict;
 use warnings;
@@ -886,9 +886,16 @@ sub munin_writeconfig_loop {
 sub munin_read_storable {
 	my ($storable_filename, $default) = @_;
 
-	return Storable::retrieve($storable_filename) if (-e $storable_filename);
+	if (-e $storable_filename) {
+		my $storable = eval { Storable::retrieve($storable_filename); };
+		return $storable unless $@;
+
+		# Didn't managed to read storable. 
+		# Removing it as it is already torched anyway.
+		unlink($storable_filename);
+	}
 	
-	# Default
+	# return default if no better option
 	return $default;
 }
 
