@@ -1,6 +1,6 @@
 package Munin::Node::Service;
 
-# $Id: Service.pm 4528 2011-12-20 17:06:06Z steve.schnepp $
+# $Id$
 
 use warnings;
 use strict;
@@ -95,6 +95,13 @@ sub prepare_plugin_environment
     # of plugins.
     $ENV{LC_ALL} = 'C';
 
+    # LC_ALL should be enough, but some plugins don't follow specs (#1014)
+    $ENV{LANG} = 'C';
+
+    # PATH should be *very* sane by default. Can be overrided via 
+    # config file if needed (Closes #863 and #1128).
+    $ENV{PATH} = '/usr/sbin:/usr/bin:/sbin:/bin';
+
     if ($config->{sconffile}) {
         # only used by munin-run
         $config->parse_plugin_config_file($config->{sconffile});
