@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
-# $Id: Plugin.pm 3955 2010-10-11 23:46:31Z bldewolf $
+# $Id$
 #
 
 package Munin::Plugin;
@@ -122,11 +122,10 @@ our $pluginstatedir = $ENV{'MUNIN_PLUGSTATE'}
 The automatically calculated name for the plugins state file.  The
 name is supplied by munin-node or munin-run (in the MUNIN_STATEFILE
 environment variable).  The file name contains the plugin name and the
-ip-number of the munin-master the node is talking to (munin-run leaves
-the master part blank) to enable stateful plugins that calculate
-gauguges and assumes a 5 minute run interval to work in munin-setups
-with multiple munin-masters (this is not a uncommon way to set up
-Munin).
+IP address of the munin-master the node is talking to (munin-run leaves
+the master part blank).  This enables stateful plugins that calculate
+gauges and assume a 5 minute run interval to work correctly in setups
+with multiple masters (this is not a uncommon way to set up Munin).
 
 To change the value of this please use the C<set_state_name($)>
 procedure (see below).
@@ -156,7 +155,7 @@ the first character must be C<[a-zA-Z_]>.  To satisfy these demands
 the function replaces illegal characters with a '_'.
 
 See also
-L<http://munin.projects.linpro.no/wiki/notes_on_datasource_names>
+L<http://munin-monitoring.org/wiki/notes_on_datasource_names>
 
 =cut
 
@@ -168,6 +167,9 @@ sub clean_fieldname ($) {
     # Replace remaining illegals with _
     $name =~ s/[^A-Za-z0-9_]/_/g;
 
+    # "root" is *not* allowed due to a 2.0 bug
+    $name = "__root" if $name eq "root";
+
     return $name;
 }
 
